# Tyk Dashboard 

Tyk Dashboard enables the management of the Tyk API Gateway server via an interface and processes analytics data for
reporting.

See [the docs](https://tyk.io/docs) and the [dev env project](https://github.com/TykTechnologies/tyk-develop-env) for
installation and configuration.

## Git submodule `webclient`

The `webclient` directory in the root of this project is a git submodule to pull in the
[`tyk-analytics-ui`](https://github.com/TykTechnologies/tyk-analytics-ui) project.

After cloning this project for the first time, initialise the submodule like so:

``` shell
$ git submodule init
Submodule 'webclient' (https://git@github.com/TykTechnologies/tyk-analytics-ui.git) registered for path 'webclient'
```

When there are new changes in the `tyk-analytics-ui` project, pull them into your local with the following command:

``` shell
$ git submodule update
Submodule path 'webclient': checked out '678157a8c367cf7474b84235c019c79a5e6a2ac6'
```

# Automation status

| Workflow | Status |
|----------|--------|
| Asset building | ![Build assets](https://github.com/TykTechnologies/tyk-analytics/workflows/Build%20assets/badge.svg?branch=master) |
| Tyk version | ![Update go.mod](https://github.com/TykTechnologies/tyk-analytics/workflows/Update%20go.mod/badge.svg?branch=master) |

## Testing goreleaser locally
Since the code requires cgo, the goreleaser configuration is split into per-architecture builds. Cross compiler toolchains are required to build. To run goreleaser locally, in the root of the repo,

``` shell
% docker run --rm --privileged -e CGO_ENABLED=1 -e GO111MODULE=on -e GITHUB_TOKEN=sekret -e PKG_SIGNING_KEY="$(<~src/rpmsign/tyk.io.signing.key)" \
       -e NFPM_STD_PASSPHRASE=sekret -e NFPM_PAYG_PASSPHRASE=sekret -e GPG_FINGERPRINT=12B5D62C28F57592D1575BD51ED14C59E37DAC20 \
       -v /var/run/docker.sock:/var/run/docker.sock -v ~/.docker/config.json:/root/.docker/config.json \
       -v `pwd`:/go/src/github.com/TykTechnologies/tyk-analytics \
       -w /go/src/github.com/TykTechnologies/tyk-analytics \
       tykio/golang-cross:1.16 bash -c 'bin/unlock-agent.sh && goreleaser --rm-dist'
```

## Setting up the toolchain
[tykio/golang-cross](https://github.com/TykTechnologies/golang-cross) contains the cross-compiler toolchains to _build_ the debs, rpms, and binary tarballs.

To build multi-arch docker manifests, use buildx and docker-ce. The experimental flag [needs to be enabled](https://docs.docker.com/engine/reference/commandline/dockerd/#description) in the docker installation. Then [setup buildx](https://github.com/docker/buildx#getting-started) and ensure that you have it configured for `linux/arm64` and `linux/amd64` platforms.

The same infrastructure is used in CI.

## Development Dockerfile

A development `Dockerfile` and `docker-compose.yml` file are provided.
They are used to facilitate docker environment based development and help
out with development tasks like testing. Local images also enable running
`docker scan` on yet unreleased images.

Requirements:

- [Enable experimental features on docker](https://thenewstack.io/how-to-enable-docker-experimental-features-and-encrypt-your-login-credentials/)
- [GitHub docs: Add a new SSH key](https://docs.github.com/en/authentication/connecting-to-github-with-ssh/adding-a-new-ssh-key-to-your-github-account)
    - The build process for Dashboard requires a private ssh key in `$HOME/.ssh/id_rsa` to access privaty repository during the build.

Run `make docker` to build the development image. This will create an
image named `internal/tyk-dashboard` which is referenced from docker
compose configuration. To override the default Go version in use, you
may set the `GO_VERSION` environment variable:

```
GO_VERSION=1.21 make docker
```

## Using Docker Compose

After building the docker image, you can use the provided
docker-compose.yml directly. Please copy and adjust `.env.example` to
`.env`. By default, you don't need to make any changes but you need to
provide `.env`. Make sure you enter the Dashboard license into
`TYK_DB_LICENSEKEY`.

Requirements:

- SSH agent or ssh key access to private TykTechnologies repositories (verify with `ssh-add -l`),
- Run `make docker` on Gateway to provide `internal/tyk-gateway`, OR
- Update Gateway image in `docker-compose.yml` to a [publicly released tyk-gateway image](https://hub.docker.com/r/tykio/tyk-gateway/tags).

Docker compose runs Gateway, Dashboard and the required services for it
to function (Redis, Mongo). Dashboard uses the example config inside the
repository, Gateway is bundled with the default.

If developers need to change the configation, they must edit the provided
`docker-compose.yml` files to configure some usual things for development:

- inject configuration for Gateway with a volume mount,
- inject settings via environment, e.g. `TYK_LOGLEVEL=debug`

A non-technical person may only need a few docker compose commands. This
is an essentials cheat sheet:

- `make docker` - builds the Dashboard docker image,
- `docker compose up` - runs the service in foreground, displaying logs,
- `docker compose up -d` - runs the service in background,
- `docker compose logs --tail=100 -f` - outputs last 100 lines of logs and new logs as they are written,
- `docker compose down` - shuts down the service running in the background.

The Makefile includes mnemonics for the essential ones: `make up`, `make logs` and `make down`.

Other similar resources showing example usage:

- [Tyk Pro Docker Demo](https://github.com/TykTechnologies/tyk-pro-docker-demo) -
  used to facilitate demoing already released docker images from docker hub,
- [Setup local environment with specific/latest code releases](https://tyktech.atlassian.net/wiki/spaces/~913760557/pages/1778155554/Setup+local+environment+with+specific+latest+code+releases)
- [Custom docker-compose configurations for testing by Tit Petric](https://github.com/TykTechnologies/titpetric/tree/main/test-cases/)
