#!/bin/bash
set -e 

# Create the data dir
rm -rf ./data
mkdir -p ./data

DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
ADMIN_KEY=""
ORG_TOKEN=""
ORG_ID=""
DASHBOARD_HOST_PORT="localhost:3000"

#ADMIN ENDPOINTS
GET_ORG="admin/organisations"

#IN_ORG ENDPOINTS
GET_APIS="api/apis/"
GET_POLICIES="api/portal/policies"
GET_CATALOGUE="api/portal/catalogue"
GET_PAGES="api/portal/pages"

for i in "$@"
do
case $i in
	-k=*|--key=*)
    ADMIN_KEY="${i#*=}"
    shift # past argument=value
    ;;
    -t=*|--token=*)
    ORG_TOKEN="${i#*=}"
    shift # past argument=value
    ;;
    -h=*|--host=*)
    DASHBOARD_HOST_PORT="${i#*=}"
    shift # past argument=value
    ;;
    -o=*|--org=*)
    ORG_ID="${i#*=}"
    shift # past argument=value
    ;;
    *)
            # unknown option
    ;;
esac
done

if [ -z "$ADMIN_KEY" ]; then
    echo "Missing admin key (-k=12345)"
    exit
fi

if [ -z "$ORG_TOKEN" ]; then
    echo "Missing Org token (-t=54321)"
    exit
fi

if [ -z "$DASHBOARD_HOST_PORT" ]; then
    echo "Missing hostname and port (-h=localhost:3000)"
    exit
fi

if [ -z "$ORG_ID" ]; then
    echo "Missing org id (-o=98765)"
    exit
fi


# Get ORG
THIS_ORG=$(curl --silent --header "admin-auth: $ADMIN_KEY" http://$DASHBOARD_HOST_PORT/$GET_ORG/$ORG_ID 2>&1)
echo $THIS_ORG > ./data/org.json
echo "Org Data in ./data/org.json"

#GET APIS
THESE_APIS=$(curl --silent --header "Authorization: $ORG_TOKEN" http://$DASHBOARD_HOST_PORT/$GET_APIS?p=-2 2>&1)
echo $THESE_APIS > ./data/apis.json
echo "API list in ./data/apis.json"

#GET POLICIES
THESE_POLICIES=$(curl --silent --header "Authorization: $ORG_TOKEN" http://$DASHBOARD_HOST_PORT/$GET_POLICIES?p=-2 2>&1)
echo $THESE_POLICIES > ./data/policies.json
echo "Policy list in ./data/policies.json"

